import { Box, InputAdornment, MenuItem, Slider, Switch, TextField } from "@mui/material";
import { useAtom } from "jotai";
import { FC } from "react";
import { defaultSettings } from "../../../../lib/settings";
import { stagedSettingsAtom } from "../../../../lib/store/jotai/settings";
import { BetaChip } from "../BetaChip";
import { CategoryTitle } from "../CategoryTitle";
import { DevChip } from "../DevChip";
import { SettingsItem } from "../SettingsItem";

interface StyleProps {
  sx?: any;
}

export const Style: FC<StyleProps> = ({ sx }) => {
  // atoms
  const [stagedSettings, setStagedSettings] = useAtom(stagedSettingsAtom);

  const handleSettingsStyleValueChange = (settingKey: string, settingValue: boolean | number | string | number[]) => {
    const newSettings = {
      ...stagedSettings,
      style: {
        ...stagedSettings.style,
        [settingKey]: settingValue,
      },
    };
    setStagedSettings(newSettings);

    return newSettings;
  };

  return (
    <Box sx={{ sx }}>
      <CategoryTitle title="Basic styles" />
      <SettingsItem
        defaultText={defaultSettings.style.dark_mode ? "On" : "Off"}
        description={
          <>
            <BetaChip />
            Dark mode
          </>
        }
        input={
          <Switch
            checked={stagedSettings.style.dark_mode}
            name="dark_mode"
            onChange={(e) => {
              handleSettingsStyleValueChange(e.target.name, e.target.checked);
            }}
          />
        }
      />
      <SettingsItem
        defaultText={defaultSettings.style.font_family}
        description="Font family"
        inputLong={
          <TextField
            fullWidth
            name="font_family"
            onChange={(e) => {
              handleSettingsStyleValueChange(e.target.name, e.target.value);
            }}
            select
            size="small"
            sx={{
              maxWidth: "100%",
            }}
            value={stagedSettings.style.font_family || "monospace"}
            variant="outlined"
          >
            <MenuItem value="monospace">Monospace</MenuItem>
            <MenuItem value="sans_serif">Sans Serif</MenuItem>
            <MenuItem value="comic_sans">Comic Sans</MenuItem>
            <MenuItem value="system_font">System font</MenuItem>
          </TextField>
        }
      />
      <SettingsItem
        defaultText={defaultSettings.style.font_scaling + "%"}
        description="Font scaling"
        inputBottom={
          <Slider
            marks={[
              {
                value: 80,
                label: "80%",
              },
              {
                value: 90,
                label: "90%",
              },
              {
                value: 100,
                label: "100%",
              },
              {
                value: 110,
                label: "110%",
              },
              {
                value: 120,
                label: "120%",
              },
            ]}
            min={80}
            max={120}
            name="font_scaling"
            onChangeCommitted={(e, value) => {
              handleSettingsStyleValueChange("font_scaling", value);
            }}
            step={10}
            value={stagedSettings.style.font_scaling}
            valueLabelDisplay="auto"
          />
        }
      />
      <CategoryTitle title="Advanced settings" />
      <SettingsItem
        defaultText={defaultSettings.style.blur_radius + "px"}
        description={
          <>
            <DevChip />
            Blur radius
          </>
        }
        input={
          <TextField
            InputProps={{
              endAdornment: <InputAdornment position="end">px</InputAdornment>,
              inputProps: { min: 0, max: 100, step: 1 },
            }}
            name="blur_radius"
            onChange={(e) => {
              handleSettingsStyleValueChange(e.target.name, parseFloat(e.target.value));
            }}
            sx={{
              width: "100%",
            }}
            size="small"
            type="number"
            value={stagedSettings.style.blur_radius}
            variant="standard"
          />
        }
      />
      <SettingsItem
        defaultText={defaultSettings.style.radius + "px"}
        description={
          <>
            <DevChip />
            Radius
          </>
        }
        input={
          <TextField
            InputProps={{
              endAdornment: <InputAdornment position="end">px</InputAdornment>,
              inputProps: { min: 0, max: 100, step: 1 },
            }}
            name="radius"
            onChange={(e) => {
              handleSettingsStyleValueChange(e.target.name, parseFloat(e.target.value));
            }}
            sx={{
              width: "100%",
            }}
            size="small"
            type="number"
            value={stagedSettings.style.radius}
            variant="standard"
          />
        }
      />
      <SettingsItem
        defaultText={defaultSettings.style.opacity.toString()}
        description={
          <>
            <DevChip />
            Opacity
          </>
        }
        input={
          <TextField
            InputProps={{
              inputProps: { min: 0, max: 1, step: 0.01 },
            }}
            name="opacity"
            onChange={(e) => {
              handleSettingsStyleValueChange(e.target.name, parseFloat(e.target.value));
            }}
            sx={{
              width: "100%",
            }}
            size="small"
            type="number"
            value={stagedSettings.style.opacity}
            variant="standard"
          />
        }
      />
      <SettingsItem
        defaultText={defaultSettings.style.transition_duration.toString()}
        description={
          <>
            <DevChip />
            Transition duration
          </>
        }
        input={
          <TextField
            InputProps={{
              endAdornment: <InputAdornment position="end">ms</InputAdornment>,
              inputProps: { min: 0, max: 100, step: 1 },
            }}
            name="transition_duration"
            onChange={(e) => {
              handleSettingsStyleValueChange(e.target.name, parseFloat(e.target.value));
            }}
            sx={{
              width: "100%",
            }}
            size="small"
            type="number"
            value={stagedSettings.style.transition_duration}
            variant="standard"
          />
        }
      />
      <SettingsItem
        defaultText={defaultSettings.style.window_height + "%"}
        description={
          <>
            <DevChip />
            Window height
          </>
        }
        input={
          <TextField
            InputProps={{
              endAdornment: <InputAdornment position="end">%</InputAdornment>,
              inputProps: { min: 0, max: 100, step: 1 },
            }}
            name="window_height"
            onChange={(e) => {
              handleSettingsStyleValueChange(e.target.name, parseFloat(e.target.value));
            }}
            sx={{
              width: "100%",
            }}
            size="small"
            type="number"
            value={stagedSettings.style.window_height}
            variant="standard"
          />
        }
      />
      <SettingsItem
        defaultText={defaultSettings.style.window_width + "px"}
        description={
          <>
            <DevChip />
            Window width
          </>
        }
        input={
          <TextField
            InputProps={{
              endAdornment: <InputAdornment position="end">px</InputAdornment>,
              inputProps: { min: 0, max: 10000, step: 1 },
            }}
            name="window_width"
            onChange={(e) => {
              handleSettingsStyleValueChange(e.target.name, parseFloat(e.target.value));
            }}
            sx={{
              width: "100%",
            }}
            size="small"
            type="number"
            value={stagedSettings.style.window_width}
            variant="standard"
          />
        }
      />
    </Box>
  );
};
